%%%% Morphing an integer-periodic signal to an irrational-period sinewave
%%%% H. Jaeger May 20, 2015
%%%% Designed using Matlab R2008b
%%%%
%%%% Note: I use suffix "PL" for "PlotList" in my telling-naming of
%%%% variables
%%%%
%%%% Note: users running later versions of Matlab reported that some
%%%% conceptor demos wouldn't succeed. The cause seems to be that my old
%%%% Matlab version uses a different (and better) implementation of the inv
%%%% function than later Matlab versions. Replacing all calls to "inv" by
%%%% calls to "pinv" so far has always resolved the problems. 

addpath('./helpers');

%%% Experiment control
randstate = 1; % random seed selector, set to any integer
newNets = 1; newSystemScalings = 1;

%%% Setting system params
N = 100; % network size
SR = 1.6; % spectral radius
NetinpScaling = SR; % scaling of pattern feeding weights
BiasScaling = 0.3; % size of bias


%%% loading 
TychonovAlpha = .0001; % regularizer for  W training
washoutLength = 100;
learnLength = 500;
signalPlotLength = 20;

%%% readout learning
TychonovAlphaReadout = 0.0001;

%%% setting the two apertures
alphas = [10 100];

%%% morphing slide settings
morphWashout = 500; % system updates before plotting starts
preMorphRecordLength = 25; % nr of steps with first conceptor 
                           % before morphing starts
morphTime = 50; % length of morph
delayMorphTime = 500; % nr of delay-embedded plot points used for the 
                      % fine line in the fingerprint dot plots
delayPlotPoints = 20; % nr of thick plot points
tN = 8; % nr of fingerprint panels


%%% Setting patterns
% first pattern
pattprofile = [0 0 3 2 5]; % the integer-periodic pattern profile, will be
                           % normalized to range [-1 1]
pattLength = size(pattprofile,2);
maxVal = max(pattprofile); minVal = min(pattprofile);
pattprofile = 2 * (pattprofile - minVal) / (maxVal - minVal) - 1;
patts{1} = @(n) (pattprofile(mod(n, pattLength)+1));
% second pattern (sine)
periodlength = 8.8352522; % setting the period length
patts{2} = @(n) sin(2 * pi * n / periodlength);


%%% Initializations

randn('state', randstate);
rand('twister', randstate);

% Create raw weights
if newNets
    if N <= 20
        Netconnectivity = 1;
    else
        Netconnectivity = 10/N;
    end
    WstarRaw = generate_internal_weights(N, Netconnectivity);
    WinRaw = randn(N, 1);
    WbiasRaw = randn(N, 1);
end

% Scale raw weights and initialize weights
if newSystemScalings
    Wstar = SR * WstarRaw;
    Win = NetinpScaling * WinRaw;
    Wbias = BiasScaling * WbiasRaw;
end



%%% load patterns

Np = 2; % nr of loaded patterns, here always = 2

% initialize data collectors
allTrainArgs = zeros(N, Np * learnLength);
allTrainOldArgs = zeros(N, Np * learnLength);
allTrainOuts = zeros(1, Np * learnLength);
patternRs = cell(1,Np);

% collect data from driving native reservoir with different drivers
for p = 1:Np
    patt = patts{p}; % current pattern generator
    xCollector = zeros(N, learnLength );
    xOldCollector = zeros(N, learnLength );
    pCollector = zeros(1, learnLength );
    x = zeros(N, 1);
    for n = 1:(washoutLength + learnLength)
        u = patt(n); % pattern input
        xOld = x;
        x = tanh(Wstar * x + Win * u + Wbias);
        if n > washoutLength
            xCollector(:, n - washoutLength ) = x;
            xOldCollector(:, n - washoutLength ) = xOld;
            pCollector(1, n - washoutLength) = u;
        end
    end
    R = xCollector * xCollector' / learnLength;    
    patternRs{p} = R;    
    
    allTrainArgs(:, (p-1)*learnLength+1:p*learnLength) = ...
        xCollector;
    allTrainOldArgs(:, (p-1)*learnLength+1:p*learnLength) = ...
        xOldCollector;
    allTrainOuts(1, (p-1)*learnLength+1:p*learnLength) = ...
        pCollector;    
end

%%% compute readout
L = Np * learnLength;
Wout = (inv(allTrainArgs * allTrainArgs' / L + ...
    TychonovAlphaReadout * eye(N)) * allTrainArgs * allTrainOuts' / L)';
% training error
NRMSE_readout = nrmse(Wout*allTrainArgs, allTrainOuts);
disp(sprintf('readout NRMSE: %g  mean abs size: %g', ...
    NRMSE_readout, mean(abs(Wout))));

%%% compute W
Wtargets = (atanh(allTrainArgs) - repmat(Wbias,1,Np*learnLength));
W = (inv(allTrainOldArgs * allTrainOldArgs'  / L + ...
    TychonovAlpha * eye(N)) * allTrainOldArgs * Wtargets' / L)';
% training errors per neuron
NRMSE_W = nrmse(W*allTrainOldArgs, Wtargets);
disp(sprintf('W mean NRMSE: %g   mean abs size  %g', ...
    mean(NRMSE_W), mean(mean(abs(W)))));

%%% compute conceptors
Cs = cell(4, Np);
for p = 1:Np
    R = patternRs{p};
    [U S V] = svd(R);
    Snew = (S * inv(S + alphas(p)^(-2) * eye(N)));    
    C = U * Snew * U';
    Cs{1, p} = C;
    Cs{2, p} = U;
    Cs{3, p} = diag(Snew);
    Cs{4, p} = diag(S);
end

%%%  morphing

ms = 0:1/morphTime:1; % the morphing slider rule
morphPL = zeros(1, morphTime);
% sinewave morphing
C1 = Cs{1,1}; C2 = Cs{1,2};
x = randn(N,1);
% washing out
m = ms(1);
for i = 1:morphWashout
    x = C1 * tanh(W * x + Wbias);
end
% initial period with pure first conceptor
preMorphPL = zeros(1,preMorphRecordLength);
m = ms(1);
for i = 1:preMorphRecordLength
    x = C1 * tanh(W * x + Wbias);
    preMorphPL(1,i) = Wout * x;
end
% morphing slide
for i = 1:morphTime
    m = ms(i);
    x = ((1-m)*C1 + m*C2) * tanh(W * x + Wbias);
    morphPL(1,i) = Wout * x;
end
% post morphem
postMorphRecordLength = preMorphRecordLength;
postMorphPL = zeros(1,postMorphRecordLength);
for i = 1:postMorphRecordLength
    x = C2 * tanh(W * x + Wbias);
    postMorphPL(1,i) = Wout * x;    
end
% assemble the three partial pattern outputs
totalMorphPL = [preMorphPL morphPL postMorphPL];

% interpolate (to be used for period length estimation in bottom panel)
interpolInc = 0.1;
L = preMorphRecordLength + morphTime + postMorphRecordLength;
interpolPoints = 1:interpolInc:L;
interpolL = length(interpolPoints);
totalMorphPLInt = ...
    interp1((1:L)', totalMorphPL', interpolPoints', 'spline');

% compute the reference period length, shown as fine line in bottom panel 
p1 = pattLength; p2 = periodlength;
pdiff = p2 - p1;
pstart = p1; pend = p1 + pdiff;
refPL = [pstart * ones(1,preMorphRecordLength), ...
    (0:(morphTime-1)) / (morphTime-1) * (pend-pstart)+pstart,...
    pend * ones(1,preMorphRecordLength)];

% estimate period lengths from the interpolated maxima distances
crestDistcounts = zeros(1, interpolL);
oldVal = 1;
counter = 0;
for i = 1:interpolL-2
    if totalMorphPLInt(i+1) - totalMorphPLInt(i) > 0 && ...
            totalMorphPLInt(i+2) - totalMorphPLInt(i+1) <= 0 &&...
            totalMorphPLInt(i) > .5
        counter = counter + 1;
        crestDistcounts(i) = counter;
        oldVal = counter;
        counter = 0;
    else
        crestDistcounts(i) = oldVal;
        counter = counter + 1;
    end
end
%subsample
crestDistcounts = ...
    crestDistcounts(1,interpolInc^(-1):interpolInc^(-1):interpolL);
crestDistcounts = crestDistcounts * interpolInc;
% patch beginning
crestDistcounts(1,1:20) = ones(1,20) * crestDistcounts(20);
% patch end
crestDistcounts = [crestDistcounts(1,1:end-5), ...
    ones(1,6) * crestDistcounts(end-5)];


% delay plot fingerprints computations
slide1 = 0 : 1/(tN-1) : 1;
delayData = zeros(tN, delayMorphTime);
x0 = rand(N,1);
for i = 1:tN
    x = x0;
    Cmix = (1-slide1(i)) * C1 + slide1(i) * C2;
    for n = 1:morphWashout
        x = Cmix * tanh(W * x + Wbias);
    end
    for n = 1:delayMorphTime
        x = Cmix * tanh(W * x + Wbias);
        delayData(i,n) = Wout * x;
    end
end
fingerPrintPoints = preMorphRecordLength + (0:tN-1)*morphTime/(tN-1);

figure(1); clf;
fs = 14; % font size
lw = 3; % line width
set(gcf, 'WindowStyle','normal');
set(gcf,'Position', [900 400 800 400]);
% color settings
c = 0.7; col1 = [1 c c]; col2 = [c c 1];
col1a = [1 0 0]; col2a = [0 0 1];

for i = 1:tN
    panelWidth = (1/(tN + 1)) * (1-0.08) ;
    panelHight = 1/(4.5);
    panelx = (1-0.08)*(i-1)*(1/tN) + ...
        (1-0.08)*(i-1)*(1/tN - panelWidth)/tN...
        + 0.04;
    panely = 2/3 + 1/20 ;
    
    subplot('Position', [panelx, panely, panelWidth, panelHight]);
    thisdata = delayData(i,1:delayPlotPoints+1);
    plot(delayData(i,1:end-1), delayData(i,2:end),'ko',...
        'MarkerSize',2, 'MarkerFaceColor', 'k');
    hold on;
    plot(thisdata(1,1:end-1), thisdata(1,2:end), 'k.',...
        'MarkerSize',25);
    hold off;
    set(gca, 'XTickLabel',[],'YTickLabel',[],...
        'XLim',[-1.4 1.4],'YLim',[-1.4 1.4],'Box','on', ...
        'Color', (1-slide1(i))*col1 + slide1(i)*col2);
end

morphStart = preMorphRecordLength;
morphEnd = preMorphRecordLength + morphTime;

subplot('Position',[0.04 1/3+0.05 1-0.08 1/3-0.05]);
hold on;
line([morphStart morphStart], [-1.2 1.2],...
    'Color', 'k', 'LineWidth', 2);
line([morphEnd morphEnd], [-1.2 1.2],...
    'Color', 'k', 'LineWidth', 2);
plot(totalMorphPL(1,1:morphStart), 'Color', col1a, 'LineWidth', lw);
slide2 = (0:(morphEnd-morphStart-1))/((morphEnd-morphStart-1));
for i = morphStart:(morphEnd-1)
    plot([i,i+1], totalMorphPL(1,i:i+1), ...
        'Color', (1-slide2(i-morphStart+1))*col1a + ...
        slide2(i-morphStart+1)*col2a, 'LineWidth', lw);
end
plot(morphEnd:(length(totalMorphPL)), ...
    totalMorphPL(1,morphEnd:end), 'Color', col2a, 'LineWidth', lw);
%plot([morphStart, morphEnd], [-1 -1], 'k.', 'MarkerSize',35);
plot(fingerPrintPoints, 1.15*ones(1,tN), 'kv', ...
    'MarkerSize',15, 'MarkerFaceColor','k');
hold off;
set(gca,  'YLim',[-1.2 1.2], 'XTick', [25 50 75 100],...
    'FontSize',fs, 'Box', 'on');

subplot('Position',[0.04 0.03 1-0.08 1/3-0.05]);
hold on;
line([morphStart morphStart], [4 10],...
    'Color', 'k', 'LineWidth', 2);
line([morphEnd morphEnd], [4 10],...
    'Color', 'k', 'LineWidth', 2);
plot(crestDistcounts, 'LineWidth',6,'Color',0.75*[1 1 1]);
plot(refPL, 'k-');
hold off;
set(gca, 'YLim', [4 ceil(periodlength+1)],...
    'XTickLabel',[], 'Box','on', 'FontSize',fs);




