%%%% demo: content-addressable memory with the aid of autoconceptors
%%%%       with quasi-periodic patterns from a 2-parametric family. Here:
%%%%       with 01-thresholding 
%%%% H. Jaeger June 17, 2015
%%%% Designed using Matlab R2008b
%%%%
%%%% Note: I use suffix "PL" for "PlotList" in my telling-naming of
%%%% variables
%%%%
%%%% Note: users running later versions of Matlab reported that some
%%%% conceptor demos wouldn't succeed. The cause seems to be that my old
%%%% Matlab version uses a different (and better) implementation of the inv
%%%% function than later Matlab versions. Replacing all calls to "inv" by
%%%% calls to "pinv" so far has always resolved the problems.

addpath('./helpers');

%%% Experiment control
randstate = 1; % the seed initializer for Matlab's random nr generator

%%% Setting system params
N = 100; SR = 1.1;
WinScaling = SR; % scaling of pattern feeding weights
biasScaling = .25;

%%% Initial C adaptation in driven runs (loading)
trainWashoutLength = 100;
learnWLength = 500;

%%% ridge regression regularizer for D 
TychonovAlphaD = .0001;

%%% ridge regression regularizer for Wout
TychonovAlphaWout = 0.0001;

%%% C learning and testing
aperture = 1000;
initialWashout = 100;
cueLength = 12;
cueNL = 0.1;

CadaptRateCue = 0.01;
CadaptRateAfterCue = 0.01; % C adaptation rate

TTs = [20 1000 10000 ]; % test times: Length of test adaptation run segments after which
                      % test diagnostics are computed. At most 3
                      % runlengths (plotting routines can't
                      % accomodate more)
SNRTest = [1 1 1]; % state signal-to-noise ratio for
               % post-cue testing episodes. Set to Inf if no noise wanted
SNRCue = Inf;

measureWashout = 50;
measureTemplateLength = 20;
measureRL = 500; % runlength for measuring output NRMSE.
% Must be at least twice measureTemplateLength
ThresholdCue = 0.01;
Thresholds = 0.1 * [1 1 1];
signalPlotLength = 15;
singValPlotLength = 20;

%%% Setting patterns
Np = 10; % Nr of patterns to be loaded and tested

%%% Initializations
randn('state', randstate);
rand('twister', randstate);
I = eye(N);


%%% Create raw weights
    WRaw = generate_internal_weights(N, 10/N);
    WinRaw = randn(N, 1);
    biasRaw = randn(N, 1);


%%% Scale raw weights and initialize trainable weights
    W = SR * WRaw;
    Win = WinScaling * WinRaw;
    bias = biasScaling * biasRaw;


%%% Set pattern handles

stretch = sqrt(30);
    for p = 1:Np
        relativePhase = rand;
        firstAmp = rand;
        patts{p} = @(n) ...
            firstAmp * sin(2 * pi * n / stretch) + ...
            (1 - firstAmp) * ...
            sin(4 * pi * (n / stretch + relativePhase));
    end


%%% pattern loading

% init data collectors 
allTrainArgs = zeros(N, Np * learnWLength);
allTrainYTargs = zeros(1, Np * learnWLength);
pTemplates = zeros(measureTemplateLength, Np);
allTrainOldArgs = zeros(N, Np * learnWLength);
allTrainDTargs = zeros(N, Np * learnWLength);

for p = 1:Np
    XCue = zeros(N,learnWLength);
    XOldCue = zeros(N,learnWLength);
    pTemplate = zeros(learnWLength,1);
    patt = patts{p};
    signValCollectCounter = 1;
    x = zeros(N, 1);
    for n = 1:trainWashoutLength
        u = patt(n);
        x =  tanh(W * x + Win * u + bias);
    end
    for n = 1: learnWLength
        u = patt(n + trainWashoutLength);
        XOldCue(:,n) = x;
        x = tanh(W * x + Win * u + bias);
        XCue(:,n) = x;
        pTemplate(n,1) = u;
    end
    pTemplates(:,p) = pTemplate(end-measureTemplateLength+1:end);
    allTrainArgs(:, (p-1)*learnWLength+1:p*learnWLength) = ...
        XCue;
    allTrainOldArgs(:, (p-1)*learnWLength+1:p*learnWLength) = ...
        XOldCue;
    allTrainDTargs(:, (p-1)*learnWLength+1:p*learnWLength) = ...
        Win * pTemplate';
    allTrainYTargs(1, (p-1)*learnWLength+1:p*learnWLength) = ...
        pTemplate;
end

% compute D
L = Np * learnWLength;
D = (inv(allTrainOldArgs * allTrainOldArgs' / L + TychonovAlphaD * I) ...
    * allTrainOldArgs * allTrainDTargs' / L)';
NRMSED = mean(nrmse(allTrainDTargs, D * allTrainOldArgs));


% compute mean variance of x, needed to compute noise sizes with
% specified SNR
varx = mean(var(allTrainArgs'));


% compute readouts
Wout = (inv(allTrainArgs * allTrainArgs' / L + TychonovAlphaWout * I) ...
    * allTrainArgs * allTrainYTargs' / L)';
NRMSEWout = nrmse(allTrainYTargs, Wout * allTrainArgs);
disp(sprintf('NRMSE Wout = %0.2g  D = %0.2g', NRMSEWout, NRMSED));

%%% test retrieval
% compute runlengths from testtimes
RLs = TTs - [0 TTs(1,1:end-1)];
yTest_PL = zeros(measureRL, Np, size(RLs,2)+1);
yTest01_PL = zeros(measureRL, Np, size(RLs,2)+1);
SV_PL = zeros(N,Np, size(RLs,2)+1);
for p = 1:Np
    fprintf('%g ',p);
    patt = patts{p};
    
    x = zeros(N,1);
    % initial preliminary C estimation from driven run
    for n = 1:initialWashout
        u = patt(n);
        x =  tanh(W * x + Win * u + bias);
    end
    C = zeros(N,N);
    b = sqrt(varx /  SNRCue);
    for n = 1:cueLength
        u = patt(n + initialWashout) + cueNL * (rand - 0.5);
        x =  tanh(W * x + Win * u + bias ) + b * randn(N,1);
        C = C + CadaptRateCue * ((x - C*x)*x' - aperture^(-2)*C);
    end
    [U S V] = svd(C);
    SV_PL(:,p,1) = diag(S);
    
    % measure quality
    xBeforeMeasure = x;
    % test with current adapted C
    for n = 1:measureWashout
        x = C * tanh(W *  x + D * x + bias);
    end    
    for n = 1:measureRL
        r = tanh(W *  x + D * x + bias);
        x = C * r;
        yTest_PL(n,p,1) = Wout * x;
    end
    % test with thresholded C
    x = xBeforeMeasure;
    [Ux Sx Vx] = svd(C);
    SxDiag = diag(Sx);
    SxDiag(SxDiag < ThresholdCue) = 0;
    SxDiag(SxDiag >= ThresholdCue) = 1;
    C01 = Ux * diag(SxDiag) * Ux';
    for n = 1:measureWashout
        x = C01 * tanh(W *  x + D * x + bias);
    end    
    for n = 1:measureRL
        x = C01 * tanh(W *  x + D * x + bias);
        yTest01_PL(n,p,1) = Wout * x;
    end
    
    x = xBeforeMeasure;
    
    % run with autoadaptation
    for i = 1:size(RLs,2)
        
        % state and noise scaling factors a and b        
        b = sqrt(varx /  SNRTest(i));
        RL = RLs(i);
        for n = 1:RL
            x = C * (tanh(W *  x + D * x + bias )+ b * randn(N,1));
            C = C + CadaptRateAfterCue * ((x - C*x)*x' - aperture^(-2)*C);            
        end
        [U S V] = svd(C);
        SV_PL(:,p,i+1) = diag(S);
        
        xBeforeMeasure = x;
        % test with current adapted C
        for n = 1:measureWashout
            x = C * tanh(W *  x + D * x + bias);
        end
        for n = 1:measureRL
            x = C * tanh(W *  x + D * x + bias);
            yTest_PL(n,p,i+1) = Wout * x;
        end
        % test with thresholded C
        x = xBeforeMeasure;
        [Ux Sx Vx] = svd(C);
        SxDiag = diag(Sx);
        SxDiag(SxDiag < ThresholdCue) = 0;
        SxDiag(SxDiag >= ThresholdCue) = 1;
        C01 = Ux * diag(SxDiag) * Ux';
        for n = 1:measureWashout
            x = C01 * tanh(W *  x + D * x + bias);
        end
        for n = 1:measureRL
            x = C01 * tanh(W *  x + D * x + bias);
            yTest01_PL(n,p, i+1) = Wout * x;
        end
        
        
        x = xBeforeMeasure;
    end
end
fprintf('\n ');
%%
% optimally align C-reconstructed readouts with drivers
intRate = 2;
pAligned_PL = zeros(Np, measureTemplateLength, size(RLs,2)+1);
NRMSEsAligned = zeros(Np, size(RLs,2)+1);


for i = 1:size(RLs,2)+1
    for p = 1:Np
        thisDriver = pTemplates(:,p);
        thisOut = yTest_PL(:,p,i);
        thisDriverInt = interp1((1:measureTemplateLength)',thisDriver,...
            (1:(1/intRate):measureTemplateLength)', 'spline')';
        thisOutInt = interp1((1:measureRL)', thisOut,...
            (1:(1/intRate):measureRL)', 'spline')';
        
        L = size(thisOutInt,2); M = size(thisDriverInt,2);
        phasematches = zeros(1,L - M);
        for phaseshift = 1:(L - M)
            phasematches(1,phaseshift) = ...
                norm(thisDriverInt - ...
                thisOutInt(1,phaseshift:phaseshift+M-1));
        end
        [maxVal maxInd] = max(-phasematches);
        pAligned_PL(p,:,i) = ...
            thisOutInt(maxInd:intRate:...
            (maxInd+intRate*measureTemplateLength-1))';
        
        NRMSEsAligned(p,i) = ...
            nrmse(pAligned_PL(p,:,i),thisDriver');
    end
end

disp(sprintf('mean NRMSEs test %s',...
    num2str((mean(NRMSEsAligned)),4)));
%%
% optimally align C01-reconstructed readouts with drivers
intRate = 2;
pAligned01_PL = zeros(Np, measureTemplateLength, size(RLs,2)+1);
NRMSEs01Aligned = zeros(Np, size(RLs,2)+1);


for i = 1:size(RLs,2)+1
    for p = 1:Np
        thisDriver = pTemplates(:,p);
        thisOut = yTest01_PL(:,p,i);
        thisDriverInt = interp1((1:measureTemplateLength)',thisDriver,...
            (1:(1/intRate):measureTemplateLength)', 'spline')';
        thisOutInt = interp1((1:measureRL)', thisOut,...
            (1:(1/intRate):measureRL)', 'spline')';
        
        L = size(thisOutInt,2); M = size(thisDriverInt,2);
        phasematches = zeros(1,L - M);
        for phaseshift = 1:(L - M)
            phasematches(1,phaseshift) = ...
                norm(thisDriverInt - ...
                thisOutInt(1,phaseshift:phaseshift+M-1));
        end
        [maxVal maxInd] = max(-phasematches);
        pAligned01_PL(p,:,i) = ...
            thisOutInt(maxInd:intRate:...
            (maxInd+intRate*measureTemplateLength-1))';
        
        NRMSEs01Aligned(p,i) = ...
            nrmse(pAligned01_PL(p,:,i),thisDriver');
    end
end

disp(sprintf('mean NRMSEs01 test %s',...
    num2str((mean(NRMSEs01Aligned)),4)));

% define color sweep
nC = size(RLs,2) + 1; % nr of colors needed
if nC == 2
    colors = [1 0 0; 0 0 1];
elseif nC == 3
    colors = [1 0 0; 0.8 0 0.8; 0 0 1];
elseif nC == 4
    colors  = [1 0 0; 1 0 0.7; 0.7 0 1; 0 0 1];
end

figure(4); clf;
fs = 16; fstext = 18;

set(gcf,'DefaultAxesColorOrder', colors);
set(gcf, 'WindowStyle','normal');
set(gcf,'Position', [900 350 400 400]);


for p = 1:min(3,Np)
    
    subplot(3,2, 2*(p-1)+1);
    hold on;
    plot(ones(1,singValPlotLength),'k--');
    plot(squeeze(SV_PL(1:singValPlotLength,p,:)), 'LineWidth',3);
    hold off;
    
    if p == 1
        title(sprintf('singular values'),'FontSize',fstext);
    end
    if p < 3
        set(gca, 'XTickLabel',[]);
    end
    set(gca, 'YLim',[0,1.2],'YTick',[0 1], 'FontSize',fs, 'Box', 'on');
    
    subplot(3,2, 2*p);
    hold on;
%     plot(squeeze(pAligned_PL(p,1:signalPlotLength,1)),...
%         'Color',  [1 .7 .7 ], 'LineWidth',8);
%     plot(squeeze(pAligned_PL(p,1:signalPlotLength,end)),...
%         'Color',  [.7 .7 1], 'LineWidth',8);
    plot(squeeze(pAligned_PL(p,1:signalPlotLength,1)),...
        'r',  'LineWidth',8);
    plot(squeeze(pAligned_PL(p,1:signalPlotLength,end)),...
        'b',   'LineWidth',8);
    plot(pTemplates(1:signalPlotLength,p), 'w', 'LineWidth',2);
    hold off;
    if p < 3
        set(gca, 'XTickLabel',[]);
    end
    if p == 1
        title(sprintf('p and y'), 'FontSize',fstext);
    end
    set(gca, 'YLim',[-1 1],'YTick',[-1 0 1], 'FontSize',fs, 'Box', 'on', ...
        'XLim', [1 signalPlotLength]);
    
end
%%
figure(5); clf;
fs = 18; fstext = 18;
set(gcf, 'WindowStyle','normal');
set(gcf,'Position', [1300 350 400 400]);
set(gcf,'DefaultAxesColorOrder', colors);
hold on;
plot(log10(NRMSEsAligned(:,1)),'s-','LineWidth',2,...
    'Color', colors(1,:), 'MarkerSize',12);
% plot(log10(NRMSEsAligned(:,2)),'x--','LineWidth',2,...
%     'Color', colors(2,:), 'MarkerSize',12);
% if size(NRMSEsAligned,2) > 2
%     plot(log10(NRMSEsAligned(:,3)),'d--','LineWidth',2, ...
%         'Color', colors(3,:), 'MarkerSize',12);
% end
if size(NRMSEsAligned,2) > 3
    plot(log10(NRMSEsAligned(:,4)),'p--','LineWidth',2, ...
        'Color', colors(4,:), 'MarkerSize',12);
end
plot(log10(NRMSEs01Aligned(:,1)),'o-','LineWidth',2,...
    'Color', 'k', 'MarkerSize',12);
% plot(log10(NRMSEs01Aligned(:,2)),'x-','LineWidth',2,...
%     'Color', colors(2,:), 'MarkerSize',12);
% if size(NRMSEsAligned,2) > 2
%     plot(log10(NRMSEs01Aligned(:,3)),'d-','LineWidth',2, ...
%         'Color', colors(3,:), 'MarkerSize',12);
% end
% if size(NRMSEs01Aligned,2) > 3
%     plot(log10(NRMSEsAligned(:,4)),'p-','LineWidth',2, ...
%         'Color', colors(4,:), 'MarkerSize',12);
% end

hold off;
set(gca,  'FontSize',fs, 'Box', 'on',...
    'XLim',[0 Np+1], 'XTick', 1:Np);
xlabel('Pattern index', 'FontSize', fs);
ylabel('log10 NRMSE', 'FontSize', fs);

